

import java.util.*;

public class RungeKutta {

public RungeKutta(){}


public static double evaluate(String s, double a, double b)
{
	Equation e1=new Equation(s);
	return e1.Evaluate(a, b);
}
public static double[] evaluaten(String []s, double a, double[] b)
{
	double[] result = b;//initialize

	for(int i=0; i<b.length; i++)
	{
		Equation e1=new Equation(s[i]);
		result[i]= e1.Evaluate(a, b[i]);
	}
	return result;
}

// Fourth order Runge-Kutta for a single ordinary differential equation
public static double Runge_Kutta(String s, double x0, double y0, double xn, double h){
double  k1 = 0.0D, k2 = 0.0D, k3 = 0.0D, k4 = 0.0D;
double  x = 0.0D, y = y0;

// Calculate nsteps
double ns = (xn - x0)/h;
ns = Math.rint(ns);
int nsteps = (int) ns;  // number of steps
h = (xn - x0)/ns;

for(int i=0; i<nsteps; i++){
	x = x0 + i*h;

	k1 = h*evaluate(s, x, y); System.out.print("f1= " +k1+"    ");
    k2 = h*evaluate(s, x + (h/2), y + (k1/2)); System.out.print("f2= " +k2+"    ");
    k3 = h*evaluate(s, x + (h/2), y + (k2/2)); System.out.print("f3= " +k3+"   ");
    k4 = h*evaluate(s, x + h, y + k3); System.out.print("f4= " +k4+"   ");

	y += k1/6 + k2/3 + k3/3 + k4/6; //Y(x+h) = Y(x) + ...
	System.out.println(y +"\n");
}
return y;
}
/*
// Fourth order Runge-Kutta for n (nequ) ordinary differential equations (ODE)
public static double[] Runge_Kuttan(String[] s, double x0, double[] y0, double xn, double h){

int nequ = y0.length;
double[] k1 =new double[nequ];
double[] k2 =new double[nequ];
double[] k3 =new double[nequ];
double[] k4 =new double[nequ];
double[] y =new double[nequ];
double[] yd =new double[nequ];
double[] dydx =new double[nequ];
double x = 0.0D;

// Calculate nsteps
double ns = (xn - x0)/h;
ns = Math.rint(ns);
int nsteps = (int) ns;
h = (xn - x0)/ns;

// initialise
for(int i=0; i<nequ; i++)y[i] = y0[i];

// iteration over allowed steps
for(int j=0; j<nsteps; j++){
	x = x0 + j*h;
	dydx = evaluaten(s, x, y);
	for(int i=0; i<nequ; i++)k1[i] = h*dydx[i];


	for(int i=0; i<nequ; i++)yd[i] = y[i] + k1[i]/2;
	dydx = evaluaten(s, x + h/2, yd);

	for(int i=0; i<nequ; i++)k2[i] = h*dydx[i];

	for(int i=0; i<nequ; i++)yd[i] = y[i] + k2[i]/2;
	dydx = evaluaten(s, x + h/2, yd);
	for(int i=0; i<nequ; i++)k3[i] = h*dydx[i];

	for(int i=0; i<nequ; i++)yd[i] = y[i] + k3[i];
	dydx = evaluaten(s, x + h, yd);
	for(int i=0; i<nequ; i++)k4[i] = h*dydx[i];

	for(int i=0; i<nequ; i++)y[i] += k1[i]/6 + k2[i]/3 + k3[i]/3 + k4[i]/6 ;


}//for
return y;
}//rungeKutta

*/

public double rk4_x(String s1, String s2, double x, double y, double dt) {
	double mx1, mx2, mx3, mx4, my1, my2, my3;
	//double dt = 0.01;
	mx1 = evaluate(s1, x,y)*dt;
	my1 = evaluate(s2, x,y)*dt;
	mx2 = evaluate(s1, x + mx1/2., y + my1/2.)*dt;
	my2 = evaluate(s2, x + mx1/2., y + my1/2.)*dt;
	mx3 = evaluate(s1, x + mx2/2., y + my2/2.)*dt;
	my3 = evaluate(s2, x + mx2/2., y + my2/2.)*dt;
	mx4 = evaluate(s1, x + mx3, y + my3)*dt;
	return x + mx1/6. + mx2/3. + mx3/3. + mx4/6.;
}

public double rk4_y(String s1, String s2, double x, double y, double dt) {
	double mx1, mx2, mx3, my1, my2, my3, my4;
	//double dt = 0.01;
	mx1 = evaluate(s1, x,y)*dt;
	my1 = evaluate(s2, x,y)*dt;
	mx2 = evaluate(s1, x + mx1/2., y + my1/2.)*dt;
	my2 = evaluate(s2, x + mx1/2., y + my1/2.)*dt;
	mx3 = evaluate(s1, x + mx2/2., y + my2/2.)*dt;
	my3 = evaluate(s2, x + mx2/2., y + my2/2.)*dt;
	my4 = evaluate(s2, x + mx3, y + my3)*dt;
	return y + my1/6. + my2/3. + my3/3. + my4/6.;
}




public static void main(String[ ] args)
{
	String sx = "2*y";
	RungeKutta rk = new RungeKutta();
	System.out.println("\n"+ rk.Runge_Kutta(sx, 0, 1, 1, 0.1));

}//main

}//class RungeKutta